import _initializerDefineProperty from "@babel/runtime/helpers/initializerDefineProperty";
import _applyDecoratedDescriptor from "@babel/runtime/helpers/applyDecoratedDescriptor";
import _initializerWarningHelper from "@babel/runtime/helpers/initializerWarningHelper";

var _dec, _dec2, _dec3, _dec4, _dec5, _dec6, _dec7, _class, _class2, _descriptor, _descriptor2, _descriptor3, _descriptor4, _descriptor5, _descriptor6, _class3, _temp;

import { DisplayObjectPool, RenderingContext, SceneGraphService, OffscreenCanvasCreator } from '@antv/g';
import { PathGeneratorFactory } from '@antv/g-plugin-canvas-renderer';
import { mat4, vec3 } from 'gl-matrix';
import { inject, injectable } from 'inversify';
export var PointInPathPickerFactory = Symbol('PointInPathPicker');
export
/**
 * pick shape(s) with Mouse/Touch event
 *
 * 1. find AABB with r-tree
 * 2. do math calculation with geometry in an accurate way
 */
var CanvasPickerPlugin = (_dec = injectable(), _dec2 = inject(SceneGraphService), _dec3 = inject(RenderingContext), _dec4 = inject(DisplayObjectPool), _dec5 = inject(OffscreenCanvasCreator), _dec6 = inject(PathGeneratorFactory), _dec7 = inject(PointInPathPickerFactory), _dec(_class = (_class2 = (_temp = _class3 = /*#__PURE__*/function () {
  function CanvasPickerPlugin() {
    var _this = this;

    _initializerDefineProperty(this, "sceneGraphService", _descriptor, this);

    _initializerDefineProperty(this, "renderingContext", _descriptor2, this);

    _initializerDefineProperty(this, "displayObjectPool", _descriptor3, this);

    _initializerDefineProperty(this, "offscreenCanvas", _descriptor4, this);

    _initializerDefineProperty(this, "pathGeneratorFactory", _descriptor5, this);

    _initializerDefineProperty(this, "pointInPathPickerFactory", _descriptor6, this);

    this.isPointInPath = function (displayObject, position) {
      var context = _this.offscreenCanvas.getOrCreateContext();

      var generatePath = _this.pathGeneratorFactory(displayObject.nodeType);

      if (generatePath) {
        generatePath(context, displayObject.attributes);
      }

      return context.isPointInPath(position.x, position.y);
    };
  }

  var _proto = CanvasPickerPlugin.prototype;

  _proto.apply = function apply(renderingService) {
    var _this2 = this;

    renderingService.hooks.pick.tap(CanvasPickerPlugin.tag, function (result) {
      // position in world space
      var _result$position = result.position,
          x = _result$position.x,
          y = _result$position.y; // query by AABB first with spatial index(r-tree)

      var rBushNodes = _this2.renderingContext.rBush.search({
        minX: x,
        minY: y,
        maxX: x,
        maxY: y
      });

      var pickedDisplayObjects = [];
      rBushNodes.forEach(function (_ref) {
        var name = _ref.name;

        var displayObject = _this2.displayObjectPool.getByName(name);

        var _displayObject$getCon = displayObject.getConfig(),
            capture = _displayObject$getCon.capture;

        if (displayObject.isVisible() && capture) {
          // use picker for current shape's type
          var pick = _this2.pointInPathPickerFactory(displayObject.nodeType);

          if (pick) {
            // invert with world matrix
            var worldTransform = displayObject.getWorldTransform();
            var invertWorldMat = mat4.invert(mat4.create(), worldTransform); // transform client position to local space, do picking in local space

            var localPosition = vec3.transformMat4(vec3.create(), vec3.fromValues(x, y, 0), invertWorldMat);

            if (pick(displayObject, {
              x: localPosition[0],
              y: localPosition[1]
            }, _this2.isPointInPath)) {
              pickedDisplayObjects.push(displayObject);
            }
          } else {
            // AABB test is enough, such as `Text`
            pickedDisplayObjects.push(displayObject);
          }
        }
      }); // TODO: find group with max z-index

      pickedDisplayObjects.sort(_this2.sceneGraphService.sort);
      return {
        position: result.position,
        // return last picked
        picked: pickedDisplayObjects[pickedDisplayObjects.length - 1]
      };
    });
  }
  /**
   * use native picking method
   * @see https://developer.mozilla.org/zh-CN/docs/Web/API/CanvasRenderingContext2D/isPointInPath
   */
  ;

  return CanvasPickerPlugin;
}(), _class3.tag = 'CanvasPickerPlugin', _temp), (_descriptor = _applyDecoratedDescriptor(_class2.prototype, "sceneGraphService", [_dec2], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor2 = _applyDecoratedDescriptor(_class2.prototype, "renderingContext", [_dec3], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor3 = _applyDecoratedDescriptor(_class2.prototype, "displayObjectPool", [_dec4], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor4 = _applyDecoratedDescriptor(_class2.prototype, "offscreenCanvas", [_dec5], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor5 = _applyDecoratedDescriptor(_class2.prototype, "pathGeneratorFactory", [_dec6], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor6 = _applyDecoratedDescriptor(_class2.prototype, "pointInPathPickerFactory", [_dec7], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
})), _class2)) || _class);
//# sourceMappingURL=CanvasPickerPlugin.js.map