"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
exports.__esModule = true;
exports.CanvasPickerPlugin = exports.PointInPathPickerFactory = void 0;
var g_1 = require("@antv/g");
var g_plugin_canvas_renderer_1 = require("@antv/g-plugin-canvas-renderer");
var gl_matrix_1 = require("gl-matrix");
var inversify_1 = require("inversify");
exports.PointInPathPickerFactory = Symbol('PointInPathPicker');
/**
 * pick shape(s) with Mouse/Touch event
 *
 * 1. find AABB with r-tree
 * 2. do math calculation with geometry in an accurate way
 */
var CanvasPickerPlugin = /** @class */ (function () {
    function CanvasPickerPlugin() {
        var _this = this;
        /**
         * use native picking method
         * @see https://developer.mozilla.org/zh-CN/docs/Web/API/CanvasRenderingContext2D/isPointInPath
         */
        this.isPointInPath = function (displayObject, position) {
            var context = _this.offscreenCanvas.getOrCreateContext();
            var generatePath = _this.pathGeneratorFactory(displayObject.nodeType);
            if (generatePath) {
                generatePath(context, displayObject.attributes);
            }
            return context.isPointInPath(position.x, position.y);
        };
    }
    CanvasPickerPlugin_1 = CanvasPickerPlugin;
    CanvasPickerPlugin.prototype.apply = function (renderingService) {
        var _this = this;
        renderingService.hooks.pick.tap(CanvasPickerPlugin_1.tag, function (result) {
            // position in world space
            var _a = result.position, x = _a.x, y = _a.y;
            // query by AABB first with spatial index(r-tree)
            var rBushNodes = _this.renderingContext.rBush.search({
                minX: x,
                minY: y,
                maxX: x,
                maxY: y
            });
            var pickedDisplayObjects = [];
            rBushNodes.forEach(function (_a) {
                var name = _a.name;
                var displayObject = _this.displayObjectPool.getByName(name);
                var capture = displayObject.getConfig().capture;
                if (displayObject.isVisible() && capture) {
                    // use picker for current shape's type
                    var pick = _this.pointInPathPickerFactory(displayObject.nodeType);
                    if (pick) {
                        // invert with world matrix
                        var worldTransform = displayObject.getWorldTransform();
                        var invertWorldMat = gl_matrix_1.mat4.invert(gl_matrix_1.mat4.create(), worldTransform);
                        // transform client position to local space, do picking in local space
                        var localPosition = gl_matrix_1.vec3.transformMat4(gl_matrix_1.vec3.create(), gl_matrix_1.vec3.fromValues(x, y, 0), invertWorldMat);
                        if (pick(displayObject, { x: localPosition[0], y: localPosition[1] }, _this.isPointInPath)) {
                            pickedDisplayObjects.push(displayObject);
                        }
                    }
                    else {
                        // AABB test is enough, such as `Text`
                        pickedDisplayObjects.push(displayObject);
                    }
                }
            });
            // TODO: find group with max z-index
            pickedDisplayObjects.sort(_this.sceneGraphService.sort);
            return {
                position: result.position,
                // return last picked
                picked: pickedDisplayObjects[pickedDisplayObjects.length - 1]
            };
        });
    };
    var CanvasPickerPlugin_1;
    CanvasPickerPlugin.tag = 'CanvasPickerPlugin';
    __decorate([
        inversify_1.inject(g_1.SceneGraphService)
    ], CanvasPickerPlugin.prototype, "sceneGraphService");
    __decorate([
        inversify_1.inject(g_1.RenderingContext)
    ], CanvasPickerPlugin.prototype, "renderingContext");
    __decorate([
        inversify_1.inject(g_1.DisplayObjectPool)
    ], CanvasPickerPlugin.prototype, "displayObjectPool");
    __decorate([
        inversify_1.inject(g_1.OffscreenCanvasCreator)
    ], CanvasPickerPlugin.prototype, "offscreenCanvas");
    __decorate([
        inversify_1.inject(g_plugin_canvas_renderer_1.PathGeneratorFactory)
    ], CanvasPickerPlugin.prototype, "pathGeneratorFactory");
    __decorate([
        inversify_1.inject(exports.PointInPathPickerFactory)
    ], CanvasPickerPlugin.prototype, "pointInPathPickerFactory");
    CanvasPickerPlugin = CanvasPickerPlugin_1 = __decorate([
        inversify_1.injectable()
    ], CanvasPickerPlugin);
    return CanvasPickerPlugin;
}());
exports.CanvasPickerPlugin = CanvasPickerPlugin;
