"use strict";
exports.__esModule = true;
exports.isPointInPath = void 0;
var gl_matrix_1 = require("gl-matrix");
var g_math_1 = require("@antv/g-math");
var math_1 = require("./utils/math");
function isPointInStroke(segments, lineWidth, px, py, length, x, y) {
    var isHit = false;
    var halfWidth = lineWidth / 2;
    for (var i = 0; i < segments.length; i++) {
        var segment = segments[i];
        var currentPoint = segment.currentPoint, params = segment.params, prePoint = segment.prePoint, box = segment.box;
        // 如果在前面已经生成过包围盒，直接按照包围盒计算
        if (box && !math_1.inBox(box.x - halfWidth, box.y - halfWidth, box.width + lineWidth, box.height + lineWidth, px, py)) {
            continue;
        }
        switch (segment.command) {
            // L 和 Z 都是直线， M 不进行拾取
            case 'L':
            case 'Z':
                isHit = math_1.inLine(prePoint[0], prePoint[1], currentPoint[0], currentPoint[1], lineWidth, px, py);
                break;
            case 'Q':
                var qDistance = g_math_1.Quad.pointDistance(prePoint[0], prePoint[1], params[1], params[2], params[3], params[4], px, py);
                isHit = qDistance <= lineWidth / 2;
                break;
            case 'C':
                var cDistance = g_math_1.Cubic.pointDistance(prePoint[0], // 上一段结束位置, 即 C 的起始点
                prePoint[1], params[1], // 'C' 的参数，1、2 为第一个控制点，3、4 为第二个控制点，5、6 为结束点
                params[2], params[3], params[4], params[5], params[6], px, py, length);
                isHit = cDistance <= lineWidth / 2;
                break;
            case 'A':
                // 计算点到椭圆圆弧的距离，暂时使用近似算法，后面可以改成切割法求最近距离
                var arcParams = segment.arcParams;
                console.log(arcParams);
                var cx = arcParams.cx, cy = arcParams.cy, rx = arcParams.rx, ry = arcParams.ry, startAngle = arcParams.startAngle, endAngle = arcParams.endAngle, xRotation = arcParams.xRotation;
                var r = rx > ry ? rx : ry;
                var scaleX = rx > ry ? 1 : rx / ry;
                var scaleY = rx > ry ? ry / rx : 1;
                // const m = transform(null, [
                //   ['t', -cx, -cy],
                //   ['r', -xRotation],
                //   ['s', 1 / scaleX, 1 / scaleY],
                // ]);
                var m = gl_matrix_1.mat3.create();
                gl_matrix_1.mat3.fromTranslation(m, [-cx, -cy]);
                gl_matrix_1.mat3.multiply(m, gl_matrix_1.mat3.fromRotation(gl_matrix_1.mat3.create(), -xRotation), m);
                gl_matrix_1.mat3.multiply(m, gl_matrix_1.mat3.fromScaling(gl_matrix_1.mat3.create(), [1 / scaleX, 1 / scaleY]), m);
                var p = gl_matrix_1.vec3.transformMat3(gl_matrix_1.vec3.create(), gl_matrix_1.vec3.fromValues(px - x, py - y, 0), m);
                isHit = math_1.inArc(0, 0, r, startAngle, endAngle, lineWidth, p[0], p[1]);
                break;
            default:
                break;
        }
        if (isHit) {
            break;
        }
    }
    return isHit;
}
function isPointInPath(displayObject, position, isPointInPath) {
    var _a = displayObject.attributes, _b = _a.lineWidth, lineWidth = _b === void 0 ? 0 : _b, stroke = _a.stroke, fill = _a.fill, path = _a.path, segments = _a.segments, hasArc = _a.hasArc, polygons = _a.polygons, polylines = _a.polylines, totalLength = _a.totalLength, x = _a.x, y = _a.y;
    var isHit = false;
    if (stroke) {
        isHit = isPointInStroke(segments, lineWidth, position.x + x, position.y + y, totalLength, x, y);
    }
    if (!isHit && fill) {
        if (hasArc) {
            // 存在曲线时，暂时使用 canvas 的 api 计算，后续可以进行多边形切割
            isHit = isPointInPath(displayObject, position);
        }
        else {
            // 提取出来的多边形包含闭合的和非闭合的，在这里统一按照多边形处理
            isHit =
                math_1.inPolygons(polygons, position.x + x, position.y + y) || math_1.inPolygons(polylines, position.x + x, position.y + y);
        }
    }
    return isHit;
}
exports.isPointInPath = isPointInPath;
